@@ -1,5 +1,5 @@
 /*  This file is part of "reprepro"
- *  Copyright (C) 2008 Bernhard R. Link
+ *  Copyright (C) 2008,2009 Bernhard R. Link
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
  *  published by the Free Software Foundation.
@@ -31,10 +31,12 @@
 #include "files.h"
 #include "target.h"
 #include "terms.h"
+#include "termdecide.h"
 #include "dpkgversions.h"
 #include "tracking.h"
 #include "filecntl.h"
 #include "mprintf.h"
+#include "globmatch.h"
 #include "copypackages.h"
 
 struct target_package_list {
@@ -328,7 +330,7 @@ static retvalue packagelist_add(struct database *database, struct distribution *
 	return result;
 }
 
-static retvalue copy_by_func(struct package_list *list, struct database *database, struct distribution *into, struct distribution *from, component_t component_atom, architecture_t architecture_atom, packagetype_t packagetype_atom, retvalue action(struct package_list*, struct database *, struct distribution *, struct distribution *, struct target *, struct target *, void *), void *data) {
+static retvalue copy_by_func(struct package_list *list, struct database *database, struct distribution *into, struct distribution *from, const struct atomlist *components, const struct atomlist *architectures, const struct atomlist *packagetypes, retvalue action(struct package_list*, struct database *, struct distribution *, struct distribution *, struct target *, struct target *, void *), void *data) {
 	retvalue result, r;
 	struct target *origtarget, *desttarget;
 
@@ -336,7 +338,7 @@ static retvalue copy_by_func(struct package_list *list, struct database *databas
 	for( origtarget = from->targets ; origtarget != NULL ;
 			origtarget = origtarget->next ) {
 		if( !target_matches(origtarget,
-				component_atom, architecture_atom, packagetype_atom) )
+				components, architectures, packagetypes) )
 			continue;
 		desttarget = distribution_gettarget(into,
 				origtarget->component_atom,
@@ -362,6 +364,7 @@ struct namelist {
 	int argc;
 	const char **argv;
 	bool *warnedabout;
+	bool *found;
 };
 
 static retvalue by_name(struct package_list *list, struct database *database, UNUSED(struct distribution *into), UNUSED(struct distribution *from), struct target *desttarget, struct target *fromtarget, void *data) {
@@ -385,6 +388,8 @@ static retvalue by_name(struct package_list *list, struct database *database, UN
 			if( verbose >= 0 && ! d->warnedabout[j])
 				fprintf(stderr, "Hint: '%s' was listed multiple times, ignoring all but first!\n", d->argv[i]);
 			d->warnedabout[j] = true;
+			/* do not complain second is missing if we ignore it: */
+			d->found[i] = true;
 			continue;
 		}
 
@@ -404,6 +409,7 @@ static retvalue by_name(struct package_list *list, struct database *database, UN
 		RET_UPDATE(result,r);
 		if( RET_WAS_ERROR(r) )
 			break;
+		d->found[i] = true;
 	}
 	r = target_closepackagesdb(fromtarget);
 	RET_ENDUPDATE(result, r);
@@ -426,17 +432,42 @@ static void packagelist_done(struct package_list *list) {
 	}
 }
 
-retvalue copy_by_name(struct database *database, struct distribution *into, struct distribution *from, int argc, const char **argv, component_t component, architecture_t architecture, packagetype_t packagetype) {
+retvalue copy_by_name(struct database *database, struct distribution *into, struct distribution *from, int argc, const char **argv, const struct atomlist *components, const struct atomlist *architectures, const struct atomlist *packagetypes) {
 	struct package_list list;
-	struct namelist names = { argc, argv, calloc(argc, sizeof(bool)) };
+	struct namelist names = { argc, argv, calloc(argc, sizeof(bool)),
+       			 calloc(argc, sizeof(bool))};
 	retvalue r;
 
-	if( FAILEDTOALLOC(names.warnedabout) )
+	if( FAILEDTOALLOC(names.warnedabout) || FAILEDTOALLOC(names.found) ) {
+		free(names.found);
+		free(names.warnedabout);
 		return RET_ERROR_OOM;
+	}
 
 	memset(&list, 0, sizeof(list));
-	r = copy_by_func(&list, database, into, from, component, architecture, packagetype, by_name, &names);
+	r = copy_by_func(&list, database, into, from, components, architectures, packagetypes, by_name, &names);
 	free(names.warnedabout);
+	if( verbose >= 0 && !RET_WAS_ERROR(r) ) {
+		int i;
+		bool first = true;
+
+		assert(names.found != NULL);
+		for( i = 0 ; i < argc ; i++ ) {
+			if( names.found[i] )
+				continue;
+			if( first )
+				(void)fputs("Will not copy as not found: ", stderr);
+			else
+				(void)fputs(", ", stderr);
+			first = false;
+			(void)fputs(argv[i], stderr);
+		}
+		if( !first ) {
+			(void)fputc('.', stderr);
+			(void)fputc('\n', stderr);
+		}
+	}
+	free(names.found);
 	if( !RET_IS_OK(r) )
 		return r;
 	r = packagelist_add(database, into, &list, from->codename);
@@ -458,6 +489,7 @@ static retvalue by_source(struct package_list *list, struct database *database,
 		return r;
 	result = RET_NOTHING;
 	while( target_nextpackage(&iterator, &packagename, &chunk) ) {
+		int i;
 		char *source, *sourceversion;
 		architecture_t package_architecture;
 
@@ -474,8 +506,9 @@ static retvalue by_source(struct package_list *list, struct database *database,
 			free(source); free(sourceversion);
 			continue;
 		}
+		i = 0;
 		if( d->argc > 1 ) {
-			int i, c;
+			int c;
 
 			i = d->argc;
 			while( --i > 0 ) {
@@ -508,21 +541,84 @@ static retvalue by_source(struct package_list *list, struct database *database,
 		RET_UPDATE(result,r);
 		if( RET_WAS_ERROR(r) )
 			break;
+		d->found[0] = true;
+		d->found[i] = true;
 	}
 	r = target_closeiterator(&iterator);
 	RET_ENDUPDATE(result, r);
 	return result;
 }
 
-retvalue copy_by_source(struct database *database, struct distribution *into, struct distribution *from, int argc, const char **argv, component_t component, architecture_t architecture, packagetype_t packagetype) {
+retvalue copy_by_source(struct database *database, struct distribution *into, struct distribution *from, int argc, const char **argv, const struct atomlist *components, const struct atomlist *architectures, const struct atomlist *packagetypes) {
 	struct package_list list;
-	struct namelist names = { argc, argv, NULL };
+	struct namelist names = { argc, argv, NULL, calloc(argc, sizeof(bool)) };
 	retvalue r;
 
+	if( FAILEDTOALLOC(names.found) ) {
+		free(names.found);
+		return RET_ERROR_OOM;
+	}
 	memset(&list, 0, sizeof(list));
 	// TODO: implement fast way by looking at source tracking
 	// (also allow copying .changes and .logs)
-	r = copy_by_func(&list, database, into, from, component, architecture, packagetype, by_source, &names);
+	r = copy_by_func(&list, database, into, from, components, architectures, packagetypes, by_source, &names);
+	if( argc == 1 && !RET_WAS_ERROR(r) && verbose >= 0 ) {
+		assert(names.found != NULL);
+
+		if( !names.found[0] ) {
+			assert( r == RET_NOTHING );
+			fprintf(stderr,
+"Nothing to do as no package with source '%s' found!\n",
+				       argv[0]);
+			free(names.found);
+			return RET_NOTHING;
+		}
+	} else if( !RET_WAS_ERROR(r) && verbose >= 0 ) {
+		int i;
+		bool first = true, anything = false;
+
+		for( i = 1 ; i < argc ; i++ ) {
+			if( names.found[i] )
+				anything = true;
+		}
+		if( !anything ) {
+			assert( r == RET_NOTHING );
+			fprintf(stderr,
+"Nothing to do as no packages with source '%s' and a requested source version found!\n",
+				       argv[0]);
+			free(names.found);
+			return RET_NOTHING;
+		}
+		for( i = 1 ; i < argc ; i++ ) {
+			if( names.found[i] )
+				continue;
+			if( first )
+				(void)fputs("Will not copy as not found: ", stderr);
+			else
+				(void)fputs(", ", stderr);
+			first = false;
+			(void)fputs(argv[i], stderr);
+		}
+		if( !first ) {
+			(void)fputc('.', stderr);
+			(void)fputc('\n', stderr);
+		}
+		if( verbose > 5 ) {
+			(void)fputs("Found versions are: ", stderr);
+			first = true;
+			for( i = 1 ; i < argc ; i++ ) {
+				if( !names.found[i] )
+					continue;
+				if( !first )
+					(void)fputs(", ", stderr);
+				first = false;
+				(void)fputs(argv[i], stderr);
+			}
+			(void)fputc('.', stderr);
+			(void)fputc('\n', stderr);
+		}
+	}
+	free(names.found);
 	if( !RET_IS_OK(r) )
 		return r;
 	r = packagelist_add(database, into, &list, from->codename);
@@ -543,7 +639,7 @@ static retvalue by_formula(struct package_list *list, struct database *database,
 		return r;
 	result = RET_NOTHING;
 	while( target_nextpackage(&iterator, &packagename, &chunk) ) {
-		r = term_decidechunk(condition, chunk);
+		r = term_decidechunktarget(condition, chunk, desttarget );
 		if( r == RET_NOTHING )
 			continue;
 		if( RET_WAS_ERROR(r) ) {
@@ -566,19 +662,63 @@ static retvalue by_formula(struct package_list *list, struct database *database,
 	return result;
 }
 
-retvalue copy_by_formula(struct database *database, struct distribution *into, struct distribution *from, const char *filter, component_t component, architecture_t architecture, packagetype_t packagetype) {
+static retvalue by_glob(struct package_list *list, struct database *database, UNUSED(struct distribution *into), UNUSED(struct distribution *from), struct target *desttarget, struct target *fromtarget, void *data) {
+	const char *glob = data;
+	struct target_cursor iterator IFSTUPIDCC(=TARGET_CURSOR_ZERO);
+	const char *packagename, *chunk;
+	architecture_t package_architecture;
+	retvalue result, r;
+
+	r = target_openiterator(fromtarget, database, READONLY, &iterator);
+	assert( r != RET_NOTHING );
+	if( !RET_IS_OK(r) )
+		return r;
+	result = RET_NOTHING;
+	while( target_nextpackage(&iterator, &packagename, &chunk) ) {
+		if( !globmatch(packagename, glob) )
+			continue;
+		r = fromtarget->getarchitecture(chunk, &package_architecture);
+		if( RET_WAS_ERROR(r) ) {
+			result = r;
+			break;
+		}
+		r = list_prepareadd(database, list, desttarget,
+				packagename, NULL, package_architecture, chunk);
+		RET_UPDATE(result,r);
+		if( RET_WAS_ERROR(r) )
+			break;
+	}
+	r = target_closeiterator(&iterator);
+	RET_ENDUPDATE(result, r);
+	return result;
+}
+
+retvalue copy_by_glob(struct database *database, struct distribution *into, struct distribution *from, const char *glob, const struct atomlist *components, const struct atomlist *architectures, const struct atomlist *packagetypes) {
+	struct package_list list;
+	retvalue r;
+
+	memset(&list, 0, sizeof(list));
+
+	r = copy_by_func(&list, database, into, from, components, architectures, packagetypes, by_glob, (void*)glob);
+	if( !RET_IS_OK(r) )
+		return r;
+	r = packagelist_add(database, into, &list, from->codename);
+	packagelist_done(&list);
+	return r;
+}
+
+retvalue copy_by_formula(struct database *database, struct distribution *into, struct distribution *from, const char *filter, const struct atomlist *components, const struct atomlist *architectures, const struct atomlist *packagetypes) {
 	struct package_list list;
 	term *condition;
 	retvalue r;
 
 	memset(&list, 0, sizeof(list));
 
-	r = term_compile(&condition, filter,
-			T_OR|T_BRACKETS|T_NEGATION|T_VERSION|T_NOTEQUAL);
+	r = term_compilefortargetdecision(&condition, filter);
 	if( !RET_IS_OK(r) ) {
 		return r;
 	}
-	r = copy_by_func(&list, database, into, from, component, architecture, packagetype, by_formula, condition);
+	r = copy_by_func(&list, database, into, from, components, architectures, packagetypes, by_formula, condition);
 	term_free(condition);
 	if( !RET_IS_OK(r) )
 		return r;
@@ -643,10 +783,19 @@ static retvalue choose_by_source(struct target *target, const char *packagename,
 	return RET_OK;
 }
 
-static retvalue choose_by_condition(UNUSED(struct target *target), UNUSED(const char *packagename), UNUSED(const char *version), const char *chunk, void *privdata) {
+static retvalue choose_by_condition(struct target *target, UNUSED(const char *packagename), UNUSED(const char *version), const char *chunk, void *privdata) {
 	term *condition = privdata;
 
-	return term_decidechunk(condition, chunk);
+	return term_decidechunktarget(condition, chunk, target);
+}
+
+static retvalue choose_by_glob(UNUSED(struct target *target), const char *packagename, UNUSED(const char *version), UNUSED(const char *chunk), void *privdata) {
+	const char *glob = privdata;
+
+	if( globmatch(packagename, glob) )
+		return RET_OK;
+	else
+		return RET_NOTHING;
 }
 
 retvalue copy_from_file(struct database *database, struct distribution *into, component_t component_atom, architecture_t architecture_atom, packagetype_t packagetype_atom, const char *filename, int argc, const char **argv) {
@@ -654,7 +803,7 @@ retvalue copy_from_file(struct database *database, struct distribution *into, co
 	retvalue result, r;
 	struct target *target;
 	struct package_list list;
-	struct namelist d = {argc, argv, NULL};
+	struct namelist d = {argc, argv, NULL, NULL};
 	char *packagename, *version;
 	architecture_t package_architecture;
 	const char *control;
@@ -719,7 +868,7 @@ retvalue copy_from_file(struct database *database, struct distribution *into, co
 
 typedef retvalue chooseaction(struct target *, const char *, const char *, const char *, void *);
 
-static retvalue restore_from_snapshot(struct database *database, struct distribution *into, component_t component_atom, architecture_t architecture_atom, packagetype_t packagetype_atom, const char *snapshotname, chooseaction action, void *d) {
+static retvalue restore_from_snapshot(struct database *database, struct distribution *into, const struct atomlist *components, const struct atomlist *architectures, const struct atomlist *packagetypes, const char *snapshotname, chooseaction action, void *d) {
 	retvalue result, r;
 	struct package_list list;
 	struct target *target;
@@ -740,7 +889,7 @@ static retvalue restore_from_snapshot(struct database *database, struct distribu
 		struct indexfile *i;
 
 		if( !target_matches(target,
-				component_atom, architecture_atom, packagetype_atom) )
+				components, architectures, packagetypes) )
 			continue;
 
 		/* we do not know what compressions where used back then
@@ -816,32 +965,37 @@ static retvalue restore_from_snapshot(struct database *database, struct distribu
 	return r;
 }
 
-retvalue restore_by_name(struct database *database, struct distribution *into, component_t component, architecture_t architecture, packagetype_t packagetype, const char *snapshotname, int argc, const char **argv) {
-	struct namelist d = {argc, argv, NULL};
+retvalue restore_by_name(struct database *database, struct distribution *into, const struct atomlist *components, const struct atomlist *architectures, const struct atomlist *packagetypes, const char *snapshotname, int argc, const char **argv) {
+	struct namelist d = {argc, argv, NULL, NULL};
 	return restore_from_snapshot(database, into,
-			component, architecture, packagetype,
+			components, architectures, packagetypes,
 			snapshotname, choose_by_name, &d);
 }
 
-retvalue restore_by_source(struct database *database, struct distribution *into, component_t component, architecture_t architecture, packagetype_t packagetype, const char *snapshotname, int argc, const char **argv) {
-	struct namelist d = {argc, argv, NULL};
+retvalue restore_by_source(struct database *database, struct distribution *into, const struct atomlist *components, const struct atomlist *architectures, const struct atomlist *packagetypes, const char *snapshotname, int argc, const char **argv) {
+	struct namelist d = {argc, argv, NULL, NULL};
 	return restore_from_snapshot(database, into,
-			component, architecture, packagetype,
+			components, architectures, packagetypes,
 			snapshotname, choose_by_source, &d);
 }
 
-retvalue restore_by_formula(struct database *database, struct distribution *into, component_t component, architecture_t architecture, packagetype_t packagetype, const char *snapshotname, const char *filter) {
+retvalue restore_by_formula(struct database *database, struct distribution *into, const struct atomlist *components, const struct atomlist *architectures, const struct atomlist *packagetypes, const char *snapshotname, const char *filter) {
 	term *condition;
 	retvalue r;
 
-	r = term_compile(&condition, filter,
-			T_OR|T_BRACKETS|T_NEGATION|T_VERSION|T_NOTEQUAL);
+	r = term_compilefortargetdecision(&condition, filter);
 	if( !RET_IS_OK(r) ) {
 		return r;
 	}
 	r = restore_from_snapshot(database, into,
-			component, architecture, packagetype,
+			components, architectures, packagetypes,
 			snapshotname, choose_by_condition, condition);
 	term_free(condition);
 	return r;
 }
+
+retvalue restore_by_glob(struct database *database, struct distribution *into, const struct atomlist *components, const struct atomlist *architectures, const struct atomlist *packagetypes, const char *snapshotname, const char *glob) {
+	return restore_from_snapshot(database, into,
+			components, architectures, packagetypes,
+			snapshotname, choose_by_glob, (void*)glob);
+}
