@@ -155,11 +155,6 @@ retvalue database_close(struct database *db) {
 		RET_UPDATE(result, r);
 		db->references = NULL;
 	}
-	if( db->oldmd5sums != NULL ) {
-		r = table_close(db->oldmd5sums);
-		RET_UPDATE(result, r);
-		db->oldmd5sums = NULL;
-	}
 	if( db->checksums != NULL ) {
 		r = table_close(db->checksums);
 		RET_UPDATE(result, r);
@@ -237,20 +232,13 @@ static retvalue database_opentable(UNUSED(struct database *database), const char
 		}
 	}
 
-#if LIBDB_VERSION == 46
-#define DB_OPEN(database,filename,name,type,flags) database->open(database,NULL,filename,name,type,flags,0664)
-#elif LIBDB_VERSION == 47
-#warning libdb4.7 not yet tested. Use on your own risk
-#define DB_OPEN(database,filename,name,type,flags) database->open(database,NULL,filename,name,type,flags,0664)
-#elif LIBDB_VERSION == 44
-#define DB_OPEN(database,filename,name,type,flags) database->open(database,NULL,filename,name,type,flags,0664)
-#elif LIBDB_VERSION == 43
+#if DB_VERSION_MAJOR == 4
 #define DB_OPEN(database,filename,name,type,flags) database->open(database,NULL,filename,name,type,flags,0664)
 #else
-#if LIBDB_VERSION == 3
+#if DB_VERSION_MAJOR == 3
 #define DB_OPEN(database,filename,name,type,flags) database->open(database,filename,name,type,flags,0664)
 #else
-#error Unexpected LIBDB_VERSION!
+#error Unexpected DB_VERSION_MAJOR!
 #endif
 #endif
 	dbret = DB_OPEN(table, fullfilename, subtable, types[type], flags);
@@ -428,16 +416,16 @@ static retvalue warnidentifers(struct database *db, const struct strlist *identi
 		memset(architecture_existed, 0, sizeof(architecture_existed));
 
 		for( t = d->targets; t != NULL ; t = t->next ) {
-			int i;
+			int o;
 
 			if( !t->existed )
 				continue;
 
-			i = atomlist_ofs(&d->architectures,
+			o = atomlist_ofs(&d->architectures,
 					t->architecture_atom);
-			assert( i >= 0 );
-			if( i >= 0 ) {
-				architecture_existed[i] = true;
+			assert( o >= 0 );
+			if( o >= 0 ) {
+				architecture_existed[o] = true;
 				/* only warn about new ones if there
 				 * is at least one old one, otherwise
 				 * it's just a new distribution */
@@ -624,11 +612,6 @@ static retvalue readversionfile(struct database *db, bool nopackagesyet) {
 		return r;
 	if( c >= 0 )
 		db->capabilities.createnewtables = true;
-	r = dpkgversions_cmp(db->lastsupportedversion, "3.3.0", &c);
-	if( RET_WAS_ERROR(r) )
-		return r;
-	if( c >= 0 )
-		db->capabilities.nomd5legacy = true;
 
 	/* ensure we can understand it */
 
@@ -696,11 +679,8 @@ static retvalue writeversionfile(struct database *db) {
 		(void)fputc('\n', f);
 	}
 	if( db->lastsupportedversion == NULL ) {
-		if( db->capabilities.nomd5legacy )
-			(void)fputs("3.3.0\n", f);
-		else
-			(void)fputs("0\n", f);
-	} else if( db->capabilities.nomd5legacy ) {
+		(void)fputs("3.3.0\n", f);
+	} else {
 		int c;
 		retvalue r;
 
@@ -711,9 +691,6 @@ static retvalue writeversionfile(struct database *db) {
 			(void)fputs(db->lastsupportedversion, f);
 			(void)fputc('\n', f);
 		}
-	} else {
-		(void)fputs(db->lastsupportedversion, f);
-		(void)fputc('\n', f);
 	}
 	if( db->dbversion == NULL )
 		fprintf(f, "bdb%d.%d.%d\n", DB_VERSION_MAJOR, DB_VERSION_MINOR, DB_VERSION_PATCH);
@@ -794,7 +771,7 @@ static retvalue createnewdatabase(struct database *db, struct distribution *dist
  * - if readonly, do not create but return with RET_NOTHING
  * - lock database, waiting a given amount of time if already locked
  */
-retvalue database_create(struct database **result, struct distribution *alldistributions, bool fast, bool nopackages, bool allowunused, bool readonly, size_t waitforlock, bool verbosedb, bool legacymd5format) {
+retvalue database_create(struct database **result, struct distribution *alldistributions, bool fast, bool nopackages, bool allowunused, bool readonly, size_t waitforlock, bool verbosedb) {
 	struct database *n;
 	retvalue r;
 	bool packagesfileexists, trackingfileexists, nopackagesyet;
@@ -817,7 +794,6 @@ retvalue database_create(struct database **result, struct distribution *alldistr
 	}
 	n->readonly = readonly;
 	n->verbose = verbosedb;
-	n->capabilities.nomd5legacy = !legacymd5format;
 
 	r = database_hasdatabasefile(n, "packages.db", &packagesfileexists);
 	if( RET_WAS_ERROR(r) )
@@ -1785,9 +1761,9 @@ retvalue database_droppackages(struct database *database, const char *identifier
 retvalue database_openfiles(struct database *db) {
 	retvalue r;
 	struct strlist identifiers;
-	bool checksumsexisted;
+	bool checksumsexisted, oldfiles;
 
-	assert( db->checksums == NULL && db->oldmd5sums == NULL );
+	assert( db->checksums == NULL );
 	assert( db->contents == NULL );
 
 	r = database_listsubtables(db, "contents.cache.db", &identifiers);
@@ -1811,28 +1787,20 @@ retvalue database_openfiles(struct database *db) {
 	assert( r != RET_NOTHING );
 	if( RET_WAS_ERROR(r) ) {
 		db->checksums = NULL;
-		db->oldmd5sums = NULL;
 		return r;
 	}
-	r = database_table(db, "files.db", "md5sums",
-			dbt_BTREE,
-			(db->capabilities.nomd5legacy || checksumsexisted)?
-				0 : DB_CREATE,
-			&db->oldmd5sums);
-	if( r == RET_NOTHING )
-		db->capabilities.nomd5legacy = true;
-	else if( RET_WAS_ERROR(r) ) {
+	r = database_hasdatabasefile(db, "files.db", &oldfiles);
+	if( RET_WAS_ERROR(r) ) {
 		(void)table_close(db->checksums);
 		db->checksums = NULL;
-		db->oldmd5sums = NULL;
 		return r;
-	} else
-		db->capabilities.nomd5legacy = false;
-
-	if( !db->capabilities.nomd5legacy )
+	}
+	if( oldfiles ) {
 		fprintf(stderr,
-"Warning: database uses deprecated format.\n"
-"Please consider running translatelegacychecksums to update to the new format.\n");
+"Error: database uses deprecated format.\n"
+"Please run translatelegacychecksums to update to the new format first.\n");
+		return RET_ERROR;
+	}
 
 	// TODO: only create this file once it is actually needed...
 	r = database_table(db, "contents.cache.db", "compressedfilelists",
@@ -1840,9 +1808,7 @@ retvalue database_openfiles(struct database *db) {
 	assert( r != RET_NOTHING );
 	if( RET_WAS_ERROR(r) ) {
 		(void)table_close(db->checksums);
-		(void)table_close(db->oldmd5sums);
 		db->checksums = NULL;
-		db->oldmd5sums = NULL;
 		db->contents = NULL;
 	}
 	return r;
@@ -2257,7 +2223,6 @@ retvalue database_translate_legacy_checksums(bool verbosedb) {
 		database_free(n);
 		return RET_ERRNO(e);
 	}
-	n->capabilities.nomd5legacy = true;
 	r = writeversionfile(n);
 	releaselock(n);
 	database_free(n);
