@@ -27,6 +27,7 @@
 #include <fcntl.h>
 #include <signal.h>
 #include <dirent.h>
+#include <time.h>
 #include "error.h"
 #include "ignore.h"
 #include "mprintf.h"
@@ -54,8 +55,6 @@
 enum permitflags {
 	/* do not error out on unused files */
 	pmf_unused_files = 0,
-	/* allow .changes file to specify multipe distributions */
-	pmf_multiple_distributions,
 	/* do not error out if there already is a newer package */
 	pmf_oldpackagenewer,
 	pmf_COUNT /* must be last */
@@ -74,11 +73,20 @@ enum cleanupflags {
 	cuf_unused_files,
 	cuf_COUNT /* must be last */
 };
+enum optionsflags {
+	/* only put _all.deb comes with those of some architecture,
+	 * only put in those architectures */
+	iof_limit_arch_all = 0,
+	/* allow .changes file to specify multipe distributions */
+	iof_multiple_distributions,
+	iof_COUNT /* must be last */
+};
 
 struct incoming {
 	/* by incoming_parse: */
 	char *name;
 	char *directory;
+	char *morguedir;
 	char *tempdir;
 	char *logdir;
 	struct strlist allow;
@@ -90,6 +98,7 @@ struct incoming {
 	bool *delete;
 	bool permit[pmf_COUNT];
 	bool cleanup[cuf_COUNT];
+	bool options[iof_COUNT];
 	/* only to ease parsing: */
 	size_t lineno;
 };
@@ -101,6 +110,7 @@ static void incoming_free(/*@only@*/ struct incoming *i) {
 	if( i == NULL )
 		return;
 	free(i->name);
+	free(i->morguedir);
 	free(i->tempdir);
 	free(i->logdir);
 	free(i->directory);
@@ -210,6 +220,15 @@ CFfinishparse(incoming) {
 		d->i = NULL;
 		return RET_ERROR;
 	}
+	if( i->morguedir != NULL && i->morguedir[0] != '/' ) {
+		char *n = calc_dirconcat(global.basedir, i->morguedir);
+		if( n == NULL ) {
+			incoming_free(i);
+			return RET_ERROR_OOM;
+		}
+		free(i->morguedir);
+		i->morguedir = n;
+	}
 	if( i->tempdir[0] != '/' ) {
 		char *n = calc_dirconcat(global.basedir, i->tempdir);
 		if( n == NULL ) {
@@ -239,6 +258,16 @@ CFfinishparse(incoming) {
 			incoming_free(i);
 			return RET_ERROR;
 	}
+	if( i->morguedir != NULL && !i->cleanup[cuf_on_deny]
+			&& !i->cleanup[cuf_on_error]
+			&& !i->cleanup[cuf_unused_files] ) {
+		fprintf(stderr,
+"Warning: There is a 'MorgueDir' but no 'Cleanup' to act on in rule '%s'\n"
+"(starting at line %u, ending at line %u of %s)!\n",
+				d->name,
+				config_firstline(iter), config_line(iter),
+				config_filename(iter));
+	}
 
 	d->i = i;
 	i->lineno = config_firstline(iter);
@@ -336,19 +365,47 @@ CFSETPROC(incoming,cleanup) {
 " (use --ignore=unknownfield to ignore this)\n");
 }
 
+CFSETPROC(incoming,options) {
+	CFSETPROCVARS(incoming,i,d);
+	static const struct constant const optionsconstants[] = {
+		{ "limit_arch_all", iof_limit_arch_all},
+		{ "multiple_distributions", iof_multiple_distributions},
+		{ NULL, -1}
+	};
+
+	if( IGNORABLE(unknownfield) )
+		return config_getflags(iter, headername, optionsconstants,
+				i->options, true, "");
+	else if( i->name == NULL )
+		return config_getflags(iter, headername, optionsconstants,
+				i->options, false,
+"\n(try put Name: before Options: to ignore if it is from the wrong rule");
+	else if( strcmp(i->name, d->name) != 0 )
+		return config_getflags(iter, headername, optionsconstants,
+				i->options, true,
+" (but not within the rule we are intrested in.)");
+	else
+		return config_getflags(iter, headername, optionsconstants,
+				i->options, false,
+" (use --ignore=unknownfield to ignore this)\n");
+}
+
 CFvalueSETPROC(incoming, name)
 CFvalueSETPROC(incoming, logdir)
 CFdirSETPROC(incoming, tempdir)
+CFdirSETPROC(incoming, morguedir)
 CFdirSETPROC(incoming, directory)
-CFtruthSETPROC2(incoming, multiple, permit[pmf_multiple_distributions])
+CFtruthSETPROC2(incoming, multiple, options[iof_multiple_distributions])
 
 static const struct configfield incomingconfigfields[] = {
 	CFr("Name", incoming, name),
 	CFr("TempDir", incoming, tempdir),
 	CFr("IncomingDir", incoming, directory),
+	CF("MorgueDir", incoming, morguedir),
 	CF("Default", incoming, default),
 	CF("Allow", incoming, allow),
 	CF("Multiple", incoming, multiple),
+	CF("Options", incoming, options),
 	CF("Cleanup", incoming, cleanup),
 	CF("Permit", incoming, permit),
 	CF("Logdir", incoming, logdir)
@@ -387,7 +444,7 @@ struct candidate {
 	/* from candidate_read */
 	int ofs;
 	char *control;
-	struct strlist keys, allkeys;
+	struct signatures *signatures;
 	/* from candidate_parse */
 	char *source, *sourceversion, *changesversion;
 	struct strlist distributions,
@@ -477,8 +534,7 @@ static void candidate_free(/*@only@*/struct candidate *c) {
 	if( c == NULL )
 		return;
 	free(c->control);
-	strlist_done(&c->keys);
-	strlist_done(&c->allkeys);
+	signatures_free(c->signatures);
 	free(c->source);
 	free(c->sourceversion);
 	free(c->changesversion);
@@ -554,7 +610,7 @@ static retvalue candidate_read(struct incoming *i, int ofs, struct candidate **r
 		return RET_ERROR_OOM;
 	}
 	n->files->ofs = n->ofs;
-	n->files->type = fe_UNKNOWN;
+	n->files->type = fe_CHANGES;
 	r = candidate_usefile(i, n, n->files);
 	assert( r != RET_NOTHING );
 	if( RET_WAS_ERROR(r) ) {
@@ -562,7 +618,7 @@ static retvalue candidate_read(struct incoming *i, int ofs, struct candidate **r
 		return r;
 	}
 	assert( n->files->tempfilename != NULL );
-	r = signature_readsignedchunk(n->files->tempfilename, BASENAME(i,ofs), &n->control, &n->keys, &n->allkeys, broken);
+	r = signature_readsignedchunk(n->files->tempfilename, BASENAME(i,ofs), &n->control, &n->signatures, broken);
 	assert( r != RET_NOTHING );
 	if( RET_WAS_ERROR(r) ) {
 		candidate_free(n);
@@ -588,15 +644,6 @@ static retvalue candidate_addfileline(struct incoming *i, struct candidate *c, c
 		free(n);
 		return r;
 	}
-	if( n->type != fe_BYHAND && n->type != fe_LOG &&
-			!atom_defined(n->architecture_atom) ) {
-		fprintf(stderr,
-"'%s' contains '%s' not matching an valid architecture in any distribution known!\n",
-				BASENAME(i,c->ofs), basefilename);
-		free(basefilename);
-		candidate_file_free(n);
-		return RET_ERROR;
-	}
 	n->ofs = strlist_ofs(&i->files, basefilename);
 	if( n->ofs < 0 ) {
 		fprintf(stderr,
@@ -764,6 +811,15 @@ static retvalue candidate_earlychecks(struct incoming *i, struct candidate *c) {
 	if( RET_WAS_ERROR(r) )
 		return r;
 	for( file = c->files ; file != NULL ; file = file->next ) {
+		if( file->type != fe_CHANGES && file->type != fe_BYHAND &&
+				file->type != fe_LOG &&
+				!atom_defined(file->architecture_atom) ) {
+			fprintf(stderr,
+"'%s' contains '%s' not matching an valid architecture in any distribution known!\n",
+					BASENAME(i, c->ofs),
+					BASENAME(i, file->ofs));
+			return RET_ERROR;
+		}
 		if( !FE_PACKAGE(file->type) )
 			continue;
 		assert( atom_defined(file->architecture_atom) );
@@ -1535,6 +1591,7 @@ static retvalue candidate_add_into(struct database *database, const struct incom
 	trackingdb tracks;
 	const char *changesfilekey = NULL;
 	char *origfilename;
+	struct atomlist binary_architectures;
 
 	if( interrupted() )
 		return RET_ERROR_INTERRUPTED;
@@ -1568,6 +1625,16 @@ static retvalue candidate_add_into(struct database *database, const struct incom
 			BASENAME(i, changesfile(c)->ofs));
 	causingfile = origfilename;
 
+	atomlist_init(&binary_architectures);
+	for( p = d->packages ; p != NULL ; p = p->next ) {
+		if( FE_BINARY(p->master->type) ) {
+			architecture_t a = p->master->architecture_atom;
+
+			if( a != architecture_all )
+				atomlist_add_uniq(&binary_architectures, a);
+		}
+	}
+
 	r = RET_OK;
 	for( p = d->packages ; p != NULL ; p = p->next ) {
 		if( p->skip ) {
@@ -1583,13 +1650,21 @@ static retvalue candidate_add_into(struct database *database, const struct incom
 					(tracks==NULL)?NULL:&trackingdata,
 					p);
 		} else if( FE_BINARY(p->master->type) ) {
+			architecture_t a = p->master->architecture_atom;
+			const struct atomlist *as, architectures = {&a, 1, 1};
+
+			if( i->options[iof_limit_arch_all] &&
+					a == architecture_all &&
+					binary_architectures.count > 0 )
+				as = &binary_architectures;
+			else
+				as = &architectures;
 			r = binaries_adddeb(&p->master->deb, database,
-					p->master->architecture_atom,
-					p->packagetype, into,
+					as, p->packagetype, into,
 					(tracks==NULL)?NULL:&trackingdata,
 					p->component_atom, &p->filekeys,
 					p->control);
-		} else if( p->master->type == fe_UNKNOWN ) {
+		} else if( p->master->type == fe_CHANGES ) {
 			/* finally add the .changes to tracking, if requested */
 			assert( p->master->name == NULL );
 			assert( tracks != NULL );
@@ -1613,13 +1688,14 @@ static retvalue candidate_add_into(struct database *database, const struct incom
 					trackingdata.pkg,
 					ft_LOG, &p->filekeys, false, database);
 		} else
-			r = RET_ERROR;
+			r = RET_ERROR_INTERNAL;
 
 		if( RET_WAS_ERROR(r) ) {
 			// TODO: remove files not yet referenced
 			break;
 		}
 	}
+	atomlist_done(&binary_architectures);
 
 	if( tracks != NULL ) {
 		retvalue r2;
@@ -1658,12 +1734,12 @@ static inline retvalue candidate_checkadd_into(struct database *database,const s
 					into, into->tracking != dt_NONE,
 					p->component_atom,
 					i->permit[pmf_oldpackagenewer]);
-		} else if( p->master->type == fe_UNKNOWN
+		} else if( p->master->type == fe_CHANGES
 				|| p->master->type == fe_BYHAND
 				|| p->master->type == fe_LOG ) {
 			continue;
 		} else
-			r = RET_ERROR;
+			r = RET_ERROR_INTERNAL;
 
 		if( RET_WAS_ERROR(r) )
 			return r;
@@ -1678,13 +1754,56 @@ static inline retvalue candidate_checkadd_into(struct database *database,const s
 		return RET_NOTHING;
 }
 
-static inline bool isallowed(UNUSED(struct incoming *i), UNUSED(struct candidate *c), UNUSED(struct distribution *into), const struct uploadpermissions *permissions) {
-	return permissions->allowall;
+static inline bool isallowed(UNUSED(struct incoming *i), struct candidate *c, UNUSED(struct distribution *into), struct upload_conditions *conditions) {
+	const struct candidate_file *file;
+
+	do switch( uploaders_nextcondition(conditions) ) {
+		case uc_ACCEPTED:
+			return true;
+		case uc_REJECTED:
+			return false;
+		case uc_SOURCENAME:
+			assert( c->source != NULL );
+			(void)uploaders_verifystring(conditions, c->source);
+			break;
+		case uc_SECTIONS:
+			for( file = c->files ; file != NULL ;
+					file = file->next ) {
+				if( !FE_PACKAGE(file->type) )
+					continue;
+				if( !uploaders_verifystring(conditions,
+							(file->section == NULL)
+							?"-":file->section) )
+					break;
+			}
+			break;
+		case uc_BINARIES:
+			for( file = c->files ; file != NULL ;
+					file = file->next ) {
+				if( !FE_BINARY(file->type) )
+					continue;
+				if( !uploaders_verifystring(conditions,
+							file->name) )
+					break;
+			}
+			break;
+		case uc_ARCHITECTURES:
+			for( file = c->files ; file != NULL ;
+					file = file->next ) {
+				if( !FE_PACKAGE(file->type) )
+					continue;
+				if( !uploaders_verifyatom(conditions,
+						file->architecture_atom) )
+					break;
+			}
+			break;
+	} while( true );
 }
 
 static retvalue candidate_checkpermissions(struct incoming *i, struct candidate *c, struct distribution *into) {
 	retvalue r;
-	int j;
+	struct upload_conditions *conditions;
+	bool allowed;
 
 	/* no rules means allowed */
 	if( into->uploaders == NULL )
@@ -1695,54 +1814,17 @@ static retvalue candidate_checkpermissions(struct incoming *i, struct candidate
 		return r;
 	assert(into->uploaderslist != NULL);
 
-	if( c->keys.count == 0 ) {
-		const struct uploadpermissions *permissions;
-
-		r = uploaders_unsignedpermissions(into->uploaderslist,
-				&permissions);
-		assert( r != RET_NOTHING );
-		if( RET_WAS_ERROR(r) )
-			return r;
-		if( permissions != NULL && isallowed(i,c,into,permissions) )
-			return RET_OK;
-	} else for( j = 0; j < c->keys.count ; j++ ) {
-		const struct uploadpermissions *permissions;
-
-		r = uploaders_permissions(into->uploaderslist,
-				c->keys.values[j], &permissions);
-		assert( r != RET_NOTHING );
-		if( RET_WAS_ERROR(r) )
-			return r;
-		if( permissions != NULL && isallowed(i,c,into,permissions) )
-			return RET_OK;
-	}
-	/* reject, check if it would have been accepted with more signatures
-	 * valid: */
-	if( verbose >= 0 &&
-	    c->allkeys.count != 0 && c->allkeys.count != c->keys.count ) {
-		for( j = 0; j < c->allkeys.count ; j++ ) {
-			const struct uploadpermissions *permissions;
-			r = uploaders_permissions(into->uploaderslist,
-					c->allkeys.values[j], &permissions);
-			assert( r != RET_NOTHING );
-			if( RET_WAS_ERROR(r) )
-				return r;
-			if( permissions != NULL &&
-			    isallowed(i,c , into, permissions) ) {
-				// TODO: get information if it was invalid because
-				// missing pub-key, expire-state or something else
-				// here so warning can be more specific.
-				fprintf(stderr,
-"'%s' would have been accepted into '%s' if signature with '%s' was checkable and valid.\n",
-					i->files.values[c->ofs],
-					into->codename,
-					c->allkeys.values[j]);
-			}
-		}
-	}
-
-	/* reject */
-	return RET_NOTHING;
+	r = uploaders_permissions(into->uploaderslist, c->signatures, &conditions);
+	assert( r != RET_NOTHING );
+	if( RET_WAS_ERROR(r) )
+		return r;
+	allowed = isallowed(i, c, into, conditions);
+	free(conditions);
+	if( allowed )
+		return RET_OK;
+	else
+		/* reject */
+		return RET_NOTHING;
 }
 
 static retvalue check_architecture_availability(const struct incoming *i, const struct candidate *c) {
@@ -1995,6 +2077,8 @@ static retvalue candidate_add(struct database *database, struct incoming *i, str
 
 	/* mark files as done */
 	for( file = c->files ; file != NULL ; file = file->next ) {
+		if( file->used )
+			i->processed[file->ofs] = true;
 		if( file->used || i->cleanup[cuf_unused_files] ) {
 			i->delete[file->ofs] = true;
 		}
@@ -2019,6 +2103,14 @@ static retvalue process_changes(struct database *database, struct incoming *i, i
 	}
 	r = candidate_earlychecks(i, c);
 	if( RET_WAS_ERROR(r) ) {
+		if( i->cleanup[cuf_on_error] ) {
+			struct candidate_file *file;
+
+			i->delete[c->ofs] = true;
+			for( file = c->files ; file != NULL ; file = file->next ) {
+				i->delete[file->ofs] = true;
+			}
+		}
 		candidate_free(c);
 		return r;
 	}
@@ -2044,7 +2136,7 @@ static retvalue process_changes(struct database *database, struct incoming *i, i
 			}
 		}
 		if( c->perdistribution != NULL &&
-				!i->permit[pmf_multiple_distributions] )
+				!i->options[iof_multiple_distributions] )
 			break;
 	}
 	if( c->perdistribution == NULL && i->default_into != NULL ) {
@@ -2072,7 +2164,7 @@ static retvalue process_changes(struct database *database, struct incoming *i, i
 					i->delete[file->ofs] = true;
 			}
 		}
-		r = RET_ERROR;
+		r = RET_ERROR_INCOMING_DENY;
 	} else {
 		if( broken ) {
 			fprintf(stderr,
@@ -2097,11 +2189,50 @@ static retvalue process_changes(struct database *database, struct incoming *i, i
 	return r;
 }
 
+static inline /*@null@*/char *create_uniq_subdir(const char *basedir) {
+	char date[16], *dir;
+	unsigned long number = 0;
+	retvalue r;
+	time_t curtime;
+	struct tm *tm;
+	int e;
+
+	r = dirs_make_recursive(basedir);
+	if( RET_WAS_ERROR(r) )
+		return NULL;
+
+	if( time(&curtime) == (time_t)-1 )
+		tm = NULL;
+	else
+		tm = gmtime(&curtime);
+	if( tm == NULL || strftime(date, 16, "%Y-%m-%d", tm) != 10 )
+		strcpy(date, "timeerror");
+
+	for( number = 0 ; number < 10000 ; number ++ ) {
+		dir = mprintf("%s/%s-%lu", basedir, date, number);
+		if( FAILEDTOALLOC(dir) )
+			return NULL;
+		if( mkdir(dir, 0777) == 0 )
+			return dir;
+		e = errno;
+		if( e != EEXIST ) {
+			fprintf(stderr, "Error %d creating directory '%s': %s\n",
+					e, dir, strerror(e));
+			free(dir);
+			return NULL;
+		}
+		free(dir);
+	}
+	fprintf(stderr, "Could not create unique subdir in '%s'!\n", basedir);
+	return NULL;
+}
+
 /* tempdir should ideally be on the same partition like the pooldir */
 retvalue process_incoming(struct database *database, struct distribution *distributions, const char *name, const char *changesfilename) {
 	struct incoming *i;
 	retvalue result,r;
 	int j;
+	char *morguedir;
 
 	result = RET_NOTHING;
 
@@ -2125,6 +2256,11 @@ retvalue process_incoming(struct database *database, struct distribution *distri
 	}
 
 	logger_wait();
+	if( i->morguedir == NULL )
+		morguedir = NULL;
+	else {
+		morguedir = create_uniq_subdir(i->morguedir);
+	}
 	for( j = 0 ; j < i->files.count ; j ++ ) {
 		char *fullfilename;
 
@@ -2136,11 +2272,38 @@ retvalue process_incoming(struct database *database, struct distribution *distri
 			result = RET_ERROR_OOM;
 			continue;
 		}
+		if( morguedir != NULL && !i->processed[j] ) {
+			char *newname = calc_dirconcat(morguedir,
+					i->files.values[j]);
+			if( newname != NULL &&
+					rename(fullfilename, newname) == 0 ) {
+				free(newname);
+				free(fullfilename);
+				continue;
+			} else if( newname == NULL ) {
+				result = RET_ERROR_OOM;
+			} else {
+				int e = errno;
+
+				fprintf(stderr,
+"Error %d moving '%s' to '%s': %s\n",		e,
+						i->files.values[j],
+						morguedir, strerror(e));
+				RET_UPDATE(result, RET_ERRNO(e));
+				/* no continue, instead
+				 * delete the file as normal: */
+			}
+		}
 		if( verbose >= 3 )
 			printf("deleting '%s'...\n", fullfilename);
 		deletefile(fullfilename);
 		free(fullfilename);
 	}
+	if( morguedir != NULL ) {
+		/* in the case it is empty, remove again */
+		(void)rmdir(morguedir);
+		free(morguedir);
+	}
 	incoming_free(i);
 	return result;
 }
