@@ -5,9 +5,9 @@
 
 typedef enum { UD_ERROR, UD_LOUDNO, UD_NO, UD_UPGRADE, UD_HOLD } upgrade_decision;
 
-typedef upgrade_decision upgrade_decide_function(void *privdata, const char *package,const char *old_version,const char *new_version,const char *newcontrolchunk);
+typedef upgrade_decision upgrade_decide_function(void *privdata, const struct target *, const char *package, const char *old_version, const char *new_version, const char *newcontrolchunk);
 
-upgrade_decision ud_always(void *privdata, const char *p,const char *ov,const char *nv,const char *nc);
+upgrade_decision ud_always(void *, const struct target *, const char *, const char *, const char *, const char *);
 
 /* The main part: */
 
